/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@since 3.3
*/

var Link = function()
{
	var link = {};
	var internal = {};
	
	// Action value for items removed from the list
	link.REMOVE = -1;
	// Action value for items already in the list
	link.EXISTING = 0;
	// Action value for items added to the list
	link.ADD = 1;
	
	/**
		Add a new section with optional initial items to a list.
		The section parameter is an object with the 'itemType'
		property set to an ITEM value and optionally a 'name' property
		if a header is desired.
		
		@param list The id or <select> list object
		@param section A section object
		@param initialItems Optional array with initial item data
		@param initialAction Action value for items added to the list,
			if not specified, 0 is assumed
	 */
	link.addSection = function(list, section, initialItems, initialAction)
	{
		list = Doc.element(list);
		list.numSections = list.numSections ? 1 : list.numSections+1;
		var itemType = section.itemType;
		
		// Add a section header if a section name is specified
		if (section.name)
		{
			var option = new Option('› '+section.name, itemType);
			option.className = 'selectoptionheader';
			option.section = section;
			list[list.length] = option;
		}
		
		// Add initial items to the section
		if (initialItems)
		{
			if (!initialAction) initialAction = Link.EXISTING;
			for (var i = 0; i < initialItems.length; i++)
			{
				var item = initialItems[i];
				// Initialize item data
				item.itemType = itemType;
				item.action = initialAction;
				item.originalValue = item.value;
				
				// Create new <select> option
				var option = new Option(item.name, item.itemType+'.'+item.id);
				// cross-reference link
				option.item = item;
				item.option = option;
				
				list[list.length] = option;
				list.knownItems[list.knownItems.length] = item;
				list.itemsById[item.itemType+'.'+item.id] = item;
			}
		}
	}
	
	/**
		Add an item to the list
	*/
	link.addItem = function(list, itemType, item)
	{
		list = Doc.element(list);
		
		var existing = list.itemsById[itemType+'.'+item.id];
		if (!existing)
		{
			// New item (cloning is needed in case 'item' is created from another window)
			item = { 'id': item.id, 
				'name': item.name,
				'itemType': itemType,
				'action': link.ADD,
				'value': item.value,
				'originalValue': item.value
				};
			
			var option = new Option(item.name, item.itemType+'.'+item.id, true, true);
			option.item = item;
			item.option = option;
			
			var insertIndex = internal.getInsertIndex(list, itemType);
			Forms.addListOption(list, insertIndex, option);			
			list.knownItems[list.knownItems.length] = item;
			list.itemsById[item.itemType+'.'+item.id] = item;
		}
		else if (existing.option)
		{
			// Selected item is already in the list
			// No change, except marking it as selected
			existing.option.selected = true;
		}
		else
		{
			// Selected item is known, but not in the list
			existing.action++;
			existing.value = item.value;
			
			var option = new Option(existing.name, existing.itemType+'.'+existing.id, true, true);
			option.item = existing;
			existing.option = option;

			var insertIndex = internal.getInsertIndex(list, itemType);
			Forms.addListOption(list, insertIndex, option);
		}
	}
	
	/**
		Remove selected items from the list.
		@param list The ID or element of the list to remove items from
		@param addToList The ID or element of another list that the removed
			items should be added to
	*/
	link.removeSelected = function(list, addToList)
	{
		list = Doc.element(list);
		var sorted = 0;
		if (addToList)
		{
			addToList = Doc.element(addToList);
			sorted = Data.get(addToList, 'is-sorted');
		}
		
		for (var i = 0; i < list.length; i++)
		{
			var item = list[i].item;
			if (list[i].selected && item && item.id)
			{
				if (addToList)
				{
					var option = new Option(item.name, item.id, false, true);
					var insertIndex = sorted ? Forms.getInsertIndexOfSortedList(addToList, option.text) : addToList.length;
					Forms.addListOption(addToList, insertIndex, option);
				}
				item.action--;
				item.value = 0;
				item.option = null;
				list[i--] = null;
			}
		}		
	}
	
	/**
		Remove all items from the given list
		@param list The ID or element of the list to remove items from
	 */
	link.removeAll = function(list)
	{
		list = Doc.element(list);
		for (var i = 0; i < list.length; i++)
		{
			var item = list[i].item;
			if (item && item.id)
			{
				item.action--;
				item.value = 0;
				item.option = null;
			}
		}
		list.length = 0;
	}
	
	/**
		Event handler for removing selected items from a list.
		The ID of the selection list should be specified in 
		'data-list-id' attribute. Optionally, the removed
		items can be added to another list by setting
		the 'data-remove-to' attribute to the ID of the other list.
	 */
	link.removeOnClick = function(event)
	{
		var listId = Data.get(event.currentTarget, 'list-id');
		var moveToListId = Data.get(event.currentTarget, 'remove-to');
		link.removeSelected(listId || event.currentTarget, moveToListId);
	}
	
	/**
		Event handler that opens a popup for selecting items to add to a
		list. The ID of the selection list should be specified
		in 'data-list-id' attribute, and the type of items to
		select should be specified in 'data-item-type' attribute.
	*/
	link.addOnClick = function(event)
	{
		var listId = Data.get(event.currentTarget, 'list-id');
		var itemType = Data.get(event.currentTarget, 'item-type');
		var exclude = Data.get(event.currentTarget, 'exclude');
		var filter = Data.get(event.currentTarget, 'filter');

		var listItems = link.getIdsInList(listId, itemType);
		if (exclude) listItems[listItems.length] = exclude;
		var url = '&exclude='+listItems.join(',');
		if (filter) url += filter;
		Dialogs.selectItem(itemType, listId, 1, url);
	}
	
	/**
		Export all ADD and REMOVE actions to hidden input fields in the same form
		as the list is part of. The hidden form fields are named as '+ITEMTYPE' and
		'-ITEMTYPE' where '+' are for added items and '-' for removed items.
		The value of each hidden field is a comma separated list of the ID values.
		@param list The id or list element
		@param actionPrefix Force the given prefix instead of auto-selecting '+' or '-'
	*/
	link.exportActions = function(list, actionPrefix)
	{
		list = Doc.element(list);
		var frm = list.form;
		
		for (var i = 0; i < list.knownItems.length; i++)
		{
			var item = list.knownItems[i];
			
			if (item.action != 0 || item.value != item.originalValue)
			{
				var prefix = actionPrefix ? actionPrefix : (item.action < 0 ? '-' : '+');
				var actionKey = prefix + item.itemType;
				if (!frm[actionKey])
				{
					Forms.addHidden(frm, actionKey, item.id);
				}
				else
				{
					frm[actionKey].value += ','+item.id;
				}
				if (item.value != undefined)
				{
					var valueKey = item.itemType+'.'+item.id;
					if (!frm[valueKey])
					{
						Forms.addHidden(frm, valueKey, item.value);
					}
					else
					{
						frm[valueKey].value = item.value;
					}
				}
			}
		}
	}
	
	/**
		Event handler for the 'base-selected' event sent by popup windows used
		for selecting items.The event.detail object should contain the the following
		properties: 'itemType', 'id' and 'name'.
	*/
	link.onSelected = function(event)
	{
		link.addItem(event.currentTarget, event.detail.itemType, event.detail);
	}
	
	/**
		Get an array with the ID values of all items
		in the list.
		@param list The ID or element of a selection list
		@param itemType Required, the type of items to get the id for
	*/
	link.getIdsInList = function(list, itemType)
	{
		list = Doc.element(list);
		var ids = [];
		for (var i = 0; i < list.length; i++)
		{
			var item = list[i].item;
			if (item && item.itemType == itemType && item.id)
			{
				ids[ids.length] = item.id;
			}
		}
		return ids;
	}
	
	/**
		Get the index where an item of the given type should be inserted into
		the list.
	*/
	internal.getInsertIndex = function(list, itemType)
	{
		list = Doc.element(list);
		if (list.numSections == 1) return list.length;

		var insertIndex = -1;
		for (var i = 0; i < list.length; i++)
		{
			var itemOrSection = list[i].item || list[i].section;
			if (itemOrSection && itemOrSection.itemType == itemType)
			{
				insertIndex = i+1;
			}
			else if (insertIndex >= 0)
			{
				return insertIndex;
			}
		}
		return list.length;
	}
	
	/**
		Element initializer method that add event handler to 'add' and 'remove'
		buttons as well as the link container itself.
	*/
	internal.addLinkHandlers = function(element, autoInit)
	{
		if (autoInit == 'add-link')
		{
			Buttons.addClickHandler(element, Link.addOnClick);
		}
		else if (autoInit == 'remove-link')
		{
			Buttons.addClickHandler(element, link.removeOnClick);
		}
		else if (autoInit == 'link-container')
		{
			internal.initializeLinkContainer(element);
			Events.addEventHandler(element, 'base-selected', link.onSelected);
			Events.addEventHandler(element, 'dblclick', link.removeOnClick);
		}
	}
	
	/**
		Initialize the link container by reading data from
		the 'data-initial-items' attribute. This should be
		an array with 'section' objects, each one defining the
		properties: 'itemType', 'name' (optional), 'items' (=array
		of initial items; optional)
	 */
	internal.initializeLinkContainer = function(list)
	{
		list.knownItems = [];
		list.itemsById = [];
		
		var sections = Data.json(list, 'initial-items');
		if (!sections) return;
		
		var initialAction = Data.int(list, 'initial-action', Link.EXISTING);
		for (var s = 0; s < sections.length; s++)
		{
			var section = sections[s];
			link.addSection(list, section, section.items, initialAction);
		}
	}
	
	
	Doc.addElementInitializer(internal.addLinkHandlers);

	return link;
}();
